! Test SAOXML5RW library
!
program test_SAOXML5RW
   use SAOXML5RW
   implicit none
   integer         :: total_records
   character(256) :: input_file='input.sao.xml', &
      output_file='output.sao.xml'

   call saoxml_open_input_file( input_file )
   if( .not. xml_ok(info) ) then
      write(*,*) 'Unable to open file ''',trim(input_file),''''
      stop
   endif
   total_records = saoxml_scan_input()
   do
      call saoxml_read_one_record()
      if( .not. xml_ok(info) ) exit  !  eof, IO error, or illegal content;  stop reading
      call show_data()
   enddo
   call saoxml_close_input_file()

   call saoxml_open_output_file( output_file )
!   This only writes the LAST saoxml record read.
   call saoxml_write_one_record()
   call saoxml_close_output_file()

end program


subroutine show_data()   
   use IonogramDerivedData
   type(IDD_TRACE) :: trace
   integer         :: i,j
   logical has_plasma_freq, has_plasma_density
   real(8) ne_local, fp_local
   ! show some metadata
   write(*,*) trim(idd%system%comments)
   if( idd%system%autoscaled ) then
      write(*,*) 'AUTOSCALED BY ', trim(idd%system%auto_scaler), ' ', trim(idd%system%auto_scaler_version)
   else 
      write(*,*) 'MANUAL SCALING BY ', trim(idd%system%manual_scaler)
   endif

   write(*,*) 'Sunspot number=', idd%geospace%sunspot_number

   ! show some characteristics
   write(*,*) 'foF2=',idd_get_char(I_FOF2), ' bound =', idd%chars(I_FOF2)%bound
   write(*,*) 'hmF2=',idd_get_char(I_ZMF2)  !  uses unfamiliar URSI name for height of F2 peak frequency
   write(*,*) 'foF1=',idd_get_char(I_FOF1)
   write(*,*) 'foEp=',idd_get_char(I_FOEP)
   write(*,*) 'foE =',idd_get_char(I_FOE)

   ! show trace data
   do i=1, IDD_MAX_TRACES
      if( idd%traces(i)%available ) then
	      write(*,*) 'Layer=', idd%traces(i)%layer, ' polarization=', &
		             idd%traces(i)%polarization, ' points=', idd%traces(i)%num
      endif
   enddo
   
   trace = idd_get_trace( VAL_LAYER_F2, 1, VAL_ORDINARY )
   if( trace%num > 0 ) then
       do i=1, trace%num
	      write(*,*) trace%frequencies(i), ' ', trace%ranges(i), ' ', trace%amplitudes(i)
       enddo
   endif

   ! show profile data
   do i=1, IDD_MAX_PROFILES
       if( idd%profiles(i)%available ) then
         ! show tabulated representation info
	      write(*,*) 'Profile type = ', trim(idd%profiles(i)%profile_type), ', tabulated at ', &
                         idd%profiles(i)%num_tabulated, ' altitudes'
         ! show chebyshev representation info
         if( idd%profiles(i)%f2_chebyshev%num > 0 ) write(*,*) &
         '  F2 Chebyshev representation with ', idd%profiles(i)%f2_chebyshev%num, ' coefficients'
         if( idd%profiles(i)%f1_chebyshev%num > 0 ) write(*,*) &
         '  F1 Chebyshev representation with ', idd%profiles(i)%f1_chebyshev%num, ' coefficients'
         if( idd%profiles(i)%e_chebyshev%num > 0 ) write(*,*) &
         '  E Chebyshev representation with ', idd%profiles(i)%e_chebyshev%num, ' coefficients'
         ! show quasi-parabolic representation info
         if( idd%profiles(i)%num_qp_segments > 0 ) then
            write(*,*) '  QP-Segment representation with ', idd%profiles(i)%num_qp_segments, ' segments'
            do j=1, idd%profiles(i)%num_qp_segments
               write(*,*) 'id=',idd%profiles(i)%qps(j)%id,': ', idd%profiles(i)%qps(j)%start_distance_km, &
               ' to ',idd%profiles(i)%qps(j)%end_distance_km
            enddo
         ! show profile valley info
         if( idd%profiles(i)%valley_width_km > 0 ) write(*,*) &
         ' Valley width and depth = ', idd%profiles(i)%valley_width_km, ':', idd%profiles(i)%valley_depth_pfu
         ! show topside profile info
         if( idd%profiles(i)%top_chap%available ) write(*,*) &
         ' Topside Chapman profile: hmF2, NmF2, Hm =', idd%profiles(i)%top_chap%peak_height_km, &
         ' :', idd%profiles(i)%top_chap%peak_density_cm3, ' :', idd%profiles(i)%top_chap%peak_scale_height_km
         endif
       endif
   enddo
   
!   Show vertical profile tables, where available
   ! show profile data
   do i=1, IDD_MAX_PROFILES
      if( idd%profiles(i)%available ) then
         if( idd%profiles(i)%profile_type == VAL_VERTICAL .and. idd%profiles(i)%num_tabulated > 0) then
	         write(*,*) 'Profile number ', i, ', tabulated at ', &
                           idd%profiles(i)%num_tabulated, ' altitudes'
		      has_plasma_freq = (size(idd%profiles(i)%plasma_frequencies) == idd%profiles(i)%num_tabulated)
		      has_plasma_density = (size(idd%profiles(i)%plasma_densities) == idd%profiles(i)%num_tabulated)
		      if( .not. has_plasma_density .and. .not. has_plasma_freq) then
		         write(*,*) 'Neither plasma frequencies nor densities tabulated in this profile'
		         cycle
		      endif
		      write(*,*)       '   Height   Fplasma   Ne/cm**3'
		      do j = 1,idd%profiles(i)%num_tabulated
		         if( has_plasma_freq) then
		            fp_local = idd%profiles(i)%plasma_frequencies(j)
		            if( has_plasma_density) then
		               ne_local = idd%profiles(i)%plasma_densities(j)
		            else
		               ne_local = idd_frequency_to_density_cm3( fp_local)
		            endif
		         else
		            ne_local = idd%profiles(i)%plasma_densities(j)
		            fp_local = idd_density_to_frequency( ne_local)
		         endif
		         write(*,'(f10.2,f10.3,1p,e12.3)') idd%profiles(i)%altitudes(j), fp_local, ne_local
		      enddo
         endif
      endif
   enddo
   

   write(*,*) idd%traces(4)%num

end subroutine show_data